<?php

class Pagerating extends Tools_Plugins_Abstract
{

    const MODE_READ_ONLY = 'readonly';

    const QUANTITY_TYPE_RATES = 'rates';

    const QUANTITY_TYPE_REVIEWS = 'reviews';

    const QUANTITY_TYPE_RATES_WITHOUT_REVIEWS = 'without-reviews';

    const DEFAULT_SYMBOLS_LIMIT = 1500;

    /**
     * Quantity types
     *
     * @var array
     */
    public static $_quantityTypes = array(
        self::QUANTITY_TYPE_RATES,
        self::QUANTITY_TYPE_REVIEWS,
        self::QUANTITY_TYPE_RATES_WITHOUT_REVIEWS
    );

    /**
     * Secure token
     */
    const PAGERATING_SECURE_TOKEN = 'PageratingToken';

    protected $_formId;

    public $dateFormatPresset = array(
        'usa'    => 'M-d-Y',
        'world'  => 'd-m-Y'
    );

    protected function _init()
    {
        $this->_layout = new Zend_Layout();
        $this->_layout->setLayoutPath(Zend_Layout::getMvcInstance()->getLayoutPath());

        if(($scriptPaths = Zend_Layout::getMvcInstance()->getView()->getScriptPaths()) !== false) {
            $this->_view->setScriptPath($scriptPaths);
        }
        $this->_view->addScriptPath(__DIR__ . '/views/');
    }

    public function beforeController()
    {
        $layout = Zend_Layout::getMvcInstance();
        $currentController = $this->_request->getParam('controller');
        if (!preg_match('~backend_~', $currentController)) {
            $layout->getView()->inlineScript()
                   ->appendFile($this->_websiteUrl . 'plugins/pagerating/system/layout/js/pagerating.min.js');
        }
    }

    /**
     * Manage config action
     */
    public function configAction()
    {
        if (Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_PLUGINS)) {
            $pageratingConfigurationMapper = Pagerating_Mappers_ConfigurationMapper::getInstance();
            if ($this->_request->isPost()) {
                $configParams = filter_var_array($this->_request->getParams(), FILTER_SANITIZE_STRING);

                $secureToken = $this->_request->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
                $tokenValid = Tools_System_Tools::validateToken($secureToken, self::PAGERATING_SECURE_TOKEN);
                if (!$tokenValid) {
                    $this->_responseHelper->fail('');
                }

                $symbilsLimit  = filter_var($this->_request->getParam('symbolsLimit'), FILTER_SANITIZE_NUMBER_INT);

                if(empty($symbilsLimit) ||  $symbilsLimit < 10) {
                    $this->_responseHelper->fail($this->_translator->translate('Limit can\'t be empty or less than 10 symbols.'));
                }

                $configParams['symbolsLimit'] = $symbilsLimit;

                $pageratingConfigurationMapper->save($configParams);

                $this->_responseHelper->success('');
            } else {
                $pageratingSettings = $pageratingConfigurationMapper->getConfigParams();
                $this->_view->settings = $pageratingSettings;
                $this->_view->translator = $this->_translator;
                $this->_layout->content = $this->_view->render('pageratingConfig.phtml');
                echo $this->_layout->render();
            }
        }
    }

    /**
     * Widget generator {$plugin:pagerating:stars[:product_id[:readonly]]}
     * @return string
     */
    protected function _makeOptionStars()
    {
        if (isset($this->_options[1]) && is_numeric($this->_options[1])) {
            $rating = Pagerating_Mappers_RatingMapper::getInstance()->getProductRatingByProdId($this->_options[1]);
        } elseif (null === ($rating = Pagerating_Mappers_RatingMapper::getInstance()->find(
                $this->_seotoasterData['id']
            ))
        ) {
            $rating = new Pagerating_Models_Rating(array('pageId' => $this->_seotoasterData['id']));
        }

        $readOnly = 'false';
        $voteLink = false;
        if (isset($this->_options[1]) && is_numeric($this->_options[1]) || in_array(
                self::MODE_READ_ONLY,
                $this->_options
            )
        ) {
            $readOnly = 'true';
            if ($rating->getRatingValue() == 0) {
                if ($page = Application_Model_Mappers_PageMapper::getInstance()->find($rating->getPageId())) {
                    $voteLink = $this->_seotoasterData['websiteUrl'] . $page->getUrl();
                }
            }
        }

        if(in_array('noschema', $this->_options)){
            $this->_view->noschema = true;
        }

        if(in_array('notext', $this->_options)){
            $this->_view->notext = true;
            $this->_view->noschema = true;
        }

        $this->_view->readOnly = $readOnly;
        $this->_view->voteLink = $voteLink;
        $this->_view->rating = $rating;

        return $this->_view->render('starrating.phtml');
    }

    protected function _makeOptionQuantity()
    {
        if (!empty($this->_options[1]) && in_array($this->_options[1], self::$_quantityTypes)) {
            $pageRatingMapper = Pagerating_Mappers_RatingMapper::getInstance();
            $reviewMapper = Pagerating_Mappers_ReviewMapper::getInstance();
            if (isset($this->_options[2]) && is_numeric($this->_options[2])) {
                $rating = $pageRatingMapper->getProductRatingByProdId($this->_options[1]);
                $ratingReview = $reviewMapper->getProductRatingByProdId($this->_options[1], '1');
            } else {
                $rating = $pageRatingMapper->find($this->_seotoasterData['id']);
                $ratingReview = $reviewMapper->findByPageId($this->_seotoasterData['id'], '1');
            }

            if ($rating === null) {
                $rating = new Pagerating_Models_Rating(array('pageId' => $this->_seotoasterData['id']));
            }

            if (empty($ratingReview)) {
                $reviewCount = 0;
            } else {
                $reviewCount = $ratingReview['reviewCount'];
            }

            $ratingCount = $rating->getRatingCount();

            if ($rating === null) {
                $rating = new Pagerating_Models_Rating(array('pageId' => $this->_seotoasterData['id']));
            }


            if ($this->_options[1] === self::QUANTITY_TYPE_RATES) {
                return $rating->getRatingCount();
            }

            if ($this->_options[1] === self::QUANTITY_TYPE_REVIEWS) {
                return $reviewCount;
            }

            if ($this->_options[1] === self::QUANTITY_TYPE_RATES_WITHOUT_REVIEWS) {
                if ($ratingCount > $reviewCount) {
                    return $ratingCount - $reviewCount;
                } else {
                    return 0;
                }
            }
        }

        return '';

    }

    /**
     * Widget generator {$plugin:pagerating:review[:moderated]}
     * @return string
     */
    protected function _makeOptionReview()
    {
        $useOldCaptcha = in_array('captcha', $this->_options);
        $configMapper = Pagerating_Mappers_ConfigurationMapper::getInstance();
        $currentConfig = $configMapper->getConfigParams();

        $isAllowed = Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_USERS);

        $ratingReviews = Pagerating_Mappers_ReviewMapper::getInstance()->getReviewsByPageId($this->_seotoasterData['id'], $isAllowed);

        $this->_view->reviews = $ratingReviews;

        if (Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_USERS) || !$this->_request->getCookie(
                'pagerating' . $this->_seotoasterData['id']
            )
        ) {
            $form = new Pagerating_Forms_Review();
            $form->setAction(
                trim($this->_websiteUrl, '/') . $this->_view->url(
                    array('name' => strtolower(__CLASS__), 'run' => 'review'),
                    'pluginroute'
                )
            );
            $this->_formId = uniqid('review-form');
            $form->setAttrib('id', $this->_formId);
            $form->setAttrib('class', 'review-form');
            $form->getElement('pageId')->setValue($this->_seotoasterData['id']);

            if(!empty($currentConfig['reviewNoCaptcha']) || !$useOldCaptcha){
                $form->removeElement('captcha');
            }
            $form->addElement(
                'text',
                'reviewFormUniq',
                array(
                    'value' => '',
                    'class' => 'hidden',
                    'id' => $this->_formId.$this->_seotoasterData['id']
                )
            );
            $form->getElement('reviewFormUniq')->removeDecorator('HtmlTag');
        } else {
            $form = null;
        }
        $rating = Pagerating_Mappers_RatingMapper::getInstance()->find(
            array('pageId = ?' => $this->_seotoasterData['id'])
        );
        if (null === $rating) {
            $rating = new Pagerating_Models_Rating(array('pageId' => $this->_seotoasterData['id']));
        }
        $this->_view->rating = $rating;

        $moderated = false;

        if(in_array('moderated', $this->_options) || !empty($currentConfig['reviewModerated'])){
            $moderated = true;
        }
        if(in_array('notmoderated', $this->_options)){
            $moderated = false;
        }

        $this->_sessionHelper->pageRatingModerated = $moderated;

        if(in_array('noschema', $this->_options)){
            $this->_view->noschema = true;
        }
        $this->_view->useRecaptcha = !$useOldCaptcha && empty($currentConfig['reviewNoCaptcha']);
        $this->_view->displayRecaptcha = false;
        if ($this->_view->useRecaptcha && !Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_PLUGINS)) {
            $this->_view->displayRecaptcha = true;
        }


        $this->_view->form = $form;

        $loggedUser = $this->_sessionHelper->getCurrentUser();
        $userRole = $loggedUser->getRoleId();

        $canEditDate = false;
        if ($userRole === Tools_Security_Acl::ROLE_ADMIN || $userRole === Tools_Security_Acl::ROLE_SUPERADMIN) {
            $canEditDate = true;
        }

        $this->_view->canEditDate = $canEditDate;

        $dateformatOptions = preg_grep('/^dateformat-.*$/', $this->_options);

        $dateFormat = 'Y-m-d';
        if(!empty($dateformatOptions)) {
            $tmpDateFormat = str_replace('dateformat-', '', $dateformatOptions);
            foreach ($tmpDateFormat as $dateOpt) {
                if(array_key_exists($dateOpt, $this->dateFormatPresset)) {
                    $dateFormat = $this->dateFormatPresset[$dateOpt];
                } elseif (!empty($dateOpt)) {
                    $dateFormat = $dateOpt;
                }
            }
        }

        $this->_view->dateFormat = $dateFormat;

        $clearDate = false;
        if(in_array('cleardate', $this->_options)) {
            $clearDate = true;
        }

        $this->_view->clearDate = $clearDate;

        return $this->_view->render('reviews.phtml');
    }

    public function ratingAction()
    {
        if ($this->_request->isPost()) {
            $pageId = filter_var($this->_request->getParam('pageid'), FILTER_VALIDATE_INT);
            $score = filter_var($this->_request->getParam('score'), FILTER_VALIDATE_FLOAT);

            if (in_array('shopping', Tools_Plugins_Tools::getEnabledPlugins(true))) {
                if ($productMapper = Models_Mapper_ProductMapper::getInstance()->findByPageId($pageId)) {
                    Zend_Controller_Action_HelperBroker::getStaticHelper('cache')->clean(
                        false,
                        false,
                        array('prodid_' . $productMapper->getId())
                    );
                }
            }

            if (!is_nan($pageId) && !is_nan($score)) {
                $status = $this->_updateRating($pageId, $score);
                if ($status) {
                    $rating = Pagerating_Mappers_RatingMapper::getInstance()->find($pageId);
                    $this->_responseHelper->success($rating->toArray());
                }
                $this->_responseHelper->fail($this->_translator->translate('You already voted'));
            }
        }
    }

    public function reviewAction()
    {
        if ($this->_request->isPost()) {
            $configMapper = Pagerating_Mappers_ConfigurationMapper::getInstance();
            $currentConfig = $configMapper->getConfigParams();

            $form = new Pagerating_Forms_Review();
            $form->addElement(
                'text',
                'reviewFormUniq',
                array(
                    'value' => '',
                    'class' => 'hidden'
                )
            );
            $form->getElement('reviewFormUniq')->removeDecorator('HtmlTag');
            if (!empty($currentConfig['reviewNoCaptcha']) || $this->_request->getParam('gRecaptchaResponse') != 'recaptchaOff') {
                $form->removeElement('captcha');
            }
            if ($this->_request->getParam('gRecaptchaResponse') != 'recaptchaOff') {
                $googleRecaptcha = new Tools_System_GoogleRecaptcha();
                $isGrecaptchaValid = $googleRecaptcha->isValid($this->_request->getParam('gRecaptchaResponse'));
            }
            $isFormValid = $form->isValid($this->_request->getPost());
            if (($isFormValid && !isset($isGrecaptchaValid)) || ($isFormValid && true === $isGrecaptchaValid)) {
                $data = $form->getValues();
                if(!isset($data['reviewFormUniq']) || $data['reviewFormUniq'] != ''){
                    $this->_responseHelper->success('');
                    return true;
                }

                if (!($page = Application_Model_Mappers_PageMapper::getInstance()->find($data['pageId']))) {
                    return json_encode(array('error' => true));
                }

                $description = filter_var($data['description'], FILTER_SANITIZE_STRING);
                $countCharacterDescription = mb_strlen($description);

                $symbolsLimit = (!empty($currentConfig['symbolsLimit'])) ? $currentConfig['symbolsLimit'] : self::DEFAULT_SYMBOLS_LIMIT;

                if($countCharacterDescription > $symbolsLimit) {
                    $this->_responseHelper->fail(array('errorField' => 'description', 'errorMessage' => $this->_translator->translate('You\'ve reached the character limit')));
                }

                $data['datePublished'] = date('Y-m-d H:i:s');

                if(empty($data['ratingValue'])) {
                    $data['ratingValue'] = '0.50';
                }

                $review = new Pagerating_Models_Review($data);
                $review->registerObserver(
                    new Tools_Mail_Watchdog(array('trigger' => Pagerating_MailWatchdog::TRIGGER_NEW_REVIEW))
                );
                $review->setVerified($this->_sessionHelper->pageRatingModerated ? 0 : 1);
                $review->setHelpful(0);

                $result = Pagerating_Mappers_ReviewMapper::getInstance()->save($review);
                if ($result) {
                    $review->notifyObservers();
                    //processing rating
                    if (!empty($data['ratingValue'])) {
                        $rating = $this->_updateRating($data['pageId'], $data['ratingValue']);
                    }
                    if ($this->_request->isXmlHttpRequest()) {
                        echo json_encode(array('error' => false));
                        return true;
                    } else {
                        echo json_encode(array('error' => false));
                    }
                }
            } else {
                $errorMessages = $form->getMessages();
                $errorMessage = '';
                $errorField = '';
                foreach ($errorMessages as $key => $message) {
                    if(empty($errorField)) {
                        $errorField = $key;
                    }
                    foreach ($message as $msg) {
                        $errorMessage .= $msg . ' ';
                    }
                }
                if (false === $isGrecaptchaValid) {
                    $errorMessage .= $this->_translator->translate('Recaptcha failed ');
                }
                $this->_responseHelper->fail(array('errorField' => $errorField, 'errorMessage' => $errorMessage));
            }
        }
    }

    public function deleteAction()
    {
        if (Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_PLUGINS) && $this->_request->isPost()) {
            $reviewId = $this->_request->getParam('reviewId');
            $reviewMapper = Pagerating_Mappers_ReviewMapper::getInstance();
            $result = ($review = $reviewMapper->find($reviewId)) ? $reviewMapper->delete($review) : 0;

            echo json_encode(array('reviewId' => $reviewId, 'result' => $result));
        }
    }

    public function publishAction()
    {
        if (Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_PLUGINS) && $this->_request->isPost()) {
            $reviewId = $this->_request->getParam('reviewId');
            $reviewMapper = Pagerating_Mappers_ReviewMapper::getInstance();
            $review = $reviewMapper->find($reviewId);

            if($review instanceof Pagerating_Models_Review) {
                $review->setVerified(1);
                $review->setHelpful(0);
                $result =  $reviewMapper->save($review);

                echo json_encode(array('reviewId' => $reviewId, 'result' => $result));
            }
        }
    }

    protected function _updateRating($pageId, $score)
    {
        $pageId = intval($pageId);

        $allowMultipleRate = false;
        if ($this->_request->getCookie('pageratingLogged' . $pageId)) {
            $allowMultipleRate = true;
        }

        if ($this->_request->getCookie('pagerating' . $pageId) && !$allowMultipleRate) {
            return false;
        }

        $score = floatval($score);

        $rating = Pagerating_Mappers_RatingMapper::getInstance()->find(array('pageId = ?' => $pageId));
        if (is_null($rating)) {
            $rating = new Pagerating_Models_Rating();
            $rating->setPageId($pageId);
        }

        $ratingCurrent = floatval($rating->getTotalPoints());
        $ratingCount = intval($rating->getRatingCount());

        $rating->setTotalPoints(round(($ratingCurrent + $score), 2));
        $rating->setRatingCount(++$ratingCount);

        if (Pagerating_Mappers_RatingMapper::getInstance()->save($rating)) {
            $s = setcookie('pagerating' . $pageId, $score, strtotime('+1 year'), '/');

            $loggedUser = $this->_sessionHelper->getCurrentUser();
            $userRole = $loggedUser->getRoleId();

            if ($userRole === Tools_Security_Acl::ROLE_ADMIN || $userRole === Tools_Security_Acl::ROLE_SUPERADMIN) {
                $s = setcookie('pageratingLogged' . $pageId, $userRole, strtotime('+1 day'), '/');
            }

            return true;
        }
        return false;
    }

    public function helpfulAction() {
        if($this->_request->isPost()) {
            $reviewId = filter_var($this->_request->getParam('reviewId'), FILTER_VALIDATE_INT);
            $reviewType = filter_var($this->_request->getParam('type'), FILTER_SANITIZE_STRING);

            $secureToken = $this->_request->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
            $tokenValid = Tools_System_Tools::validateToken($secureToken, self::PAGERATING_SECURE_TOKEN);
            if (!$tokenValid) {
                $this->_responseHelper->fail('');
            }

            if(!empty($reviewId)) {
                $reviewMapper = Pagerating_Mappers_ReviewMapper::getInstance();
                $currentReview = $reviewMapper->find($reviewId);

                if($currentReview instanceof Pagerating_Models_Review && !empty($reviewType)) {
                    $helpful = $currentReview->getHelpful();

                    if($reviewType == 'helpful') {
                        $helpful += 1;
                    }

                    $currentReview->setHelpful($helpful);

                    $reviewMapper->save($currentReview);

                    $this->_responseHelper->success(array('helpful' => $helpful));
                } else {
                    $this->_responseHelper->fail($this->_translator->translate('Can\'t add helpful rate'));
                }
            }
        }
    }

    public function replyformAction() {
        if (Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_PLUGINS) && $this->_request->isPost()) {
            $tokenToValidate = $this->_request->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
            $valid = Tools_System_Tools::validateToken($tokenToValidate, self::PAGERATING_SECURE_TOKEN);
            if (!$valid) {
                $this->_responseHelper->fail('');
            }

            $data = $this->_request->getParams();
            if(!empty($data['reviewId']) && !empty($data['reply-text'] && (!empty($data['reply-first-name']) || !empty($data['reply-last-name'])))) {
                $reviewMapper = Pagerating_Mappers_ReviewMapper::getInstance();
                $currentReview = $reviewMapper->find($data['reviewId']);
                if($currentReview instanceof Pagerating_Models_Review) {
                    $isVerified = $currentReview->getVerified();
                    if(!empty($isVerified)) {
                        $currentReview->setReplyFirstname($data['reply-first-name']);
                        $currentReview->setReplyLastname($data['reply-last-name']);
                        $currentReview->setReplyText($data['reply-text']);

                        $reviewMapper->save($currentReview);

                        $currentReview->registerObserver(
                            new Tools_Mail_Watchdog(array('trigger' => Pagerating_MailWatchdog::TRIGGER_REPLY, 'replyText' => $data['reply-text']))
                        );

                        $currentReview->notifyObservers();

                        $this->_responseHelper->success(array('replyFirstname' => $data['reply-first-name'], 'replyLastname' => $data['reply-last-name'], 'replyText' => $data['reply-text']));
                    }
                }
            } else {
                $this->_responseHelper->fail($this->_translator->translate('Can\'t send form'));
            }
        }
    }

    public function updateratingdateAction() {
        if (Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_PLUGINS) && $this->_request->isPost()) {
            $tokenToValidate = $this->_request->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
            $valid = Tools_System_Tools::validateToken($tokenToValidate, self::PAGERATING_SECURE_TOKEN);
            if (!$valid) {
                $this->_responseHelper->fail('');
            }

            $loggedUser = $this->_sessionHelper->getCurrentUser();
            $userRole = $loggedUser->getRoleId();

            if ($userRole === Tools_Security_Acl::ROLE_ADMIN || $userRole === Tools_Security_Acl::ROLE_SUPERADMIN) {
                $rId = filter_var($this->_request->getParam('rId', false), FILTER_SANITIZE_NUMBER_INT);
                $rDate = filter_var($this->_request->getParam('rDate', false), FILTER_SANITIZE_STRING);
                $dateformattoshow = filter_var($this->_request->getParam('dateformattoshow', false), FILTER_SANITIZE_STRING);

                if(!empty($rId) && !empty($rDate)) {
                    $reviewMapper = Pagerating_Mappers_ReviewMapper::getInstance();

                    $review = $reviewMapper->findByReviewId($rId);

                    if($review instanceof Pagerating_Models_Review) {
                        $oldDatePublished = $review->getDatePublished();
                        if(!empty($oldDatePublished)) {
                            $oldDatePublished = explode(' ', $oldDatePublished);
                            $oldDatePublished[0] = $rDate;

                            $oldDatePublished = implode(' ', $oldDatePublished);

                            $review->setDatePublished($oldDatePublished);
                            $reviewMapper->save($review);
                        }

                        $dateToShow = $oldDatePublished;
                        if(!empty($dateformattoshow)) {
                            $dateToShow = date($dateformattoshow, strtotime($oldDatePublished));
                        }

                        $this->_responseHelper->success(array('status' => $this->_translator->translate('Updated'), 'dateToShow' => $dateToShow));
                    }
                }
            }

            $this->_responseHelper->fail($this->_translator->translate('You can\'t update date'));
        }
    }

}

