<?php


class Pagerating_Mappers_ReviewMapper extends Application_Model_Mappers_Abstract {

	protected $_dbTable = 'Pagerating_DbTables_ReviewDbTable';

	protected $_model   = 'Pagerating_Models_Review';

	public function save($review) {
		if(!$review instanceof $this->_model) {
			throw new Exceptions_SeotoasterException('Given parameter should be ' . $this->_model . ' instance');
		}
		$data = $review->toArray();

		if ($review->getId()){
			return $this->getDbTable()->update($data, array('id = ?' => $review->getId()));
		} else {
			return $this->getDbTable()->insert($data);
		}
	}


	public function findByPageId($pageId, $verified = array())
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('pl_r.pageId = ?', $pageId);
        if (!empty($verified)) {
            $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('pl_r.verified IN (?)', $verified);
        }
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('pl_r'=>'plugin_pagerating_review'), array(new Zend_Db_Expr('COUNT(pl_r.pageId) as reviewCount')))
            ->where($where)->group('pl_r.pageId');

        return $this->getDbTable()->getAdapter()->fetchRow($select);
    }

    public function getProductRatingByProdId($productId, $verified = array()) {
        $where = $this->getDbTable()->getAdapter()->quoteInto('sh_p.id = ?', $productId);
        if (!empty($verified)) {
            $where .= ' AND '. $this->getDbTable()->getAdapter()->quoteInto('pl_r.verified IN (?)', $verified);
        }
	    $select = $this->getDbTable()->getAdapter()->select()
            ->setIntegrityCheck(false)
            ->from(array('sh_p'=>'shopping_product'), array(''))
            ->joinLeft(array('pl_r'=>'plugin_pagerating_review'), 'sh_p.page_id=pl_r.pageId', array(new Zend_Db_Expr('COUNT(pl_r.pageId) as reviewCount')))
            ->where($where)->group('pl_r.pageId');

        return $this->getDbTable()->getAdapter()->fetchRow($select);
    }

	public function delete($model){
		if(!$model instanceof $this->_model) {
			throw new Exceptions_SeotoasterException('Given parameter should be ' . $this->_model . ' instance');
		}
		$where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $model->getId());

		return $this->getDbTable()->delete($where);
	}

    public function findByReviewId($rId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $rId);
        return $this->_findWhere($where);
    }

    public function getReviewsByPageId($pageId, $isAllowed) {
        $where = $this->getDbTable()->getAdapter()->quoteInto('pageId = ?', $pageId);

        if(!$isAllowed) {
            $where .= ' AND ' . $this->getDbTable()->getAdapter()->quoteInto('verified = ?', '1');
        }

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('ppr'=>'plugin_pagerating_review'), array(
                'ppr.id',
                'ppr.pageId',
                'ppr.name',
                'ppr.author',
                'ppr.email',
                'ppr.description',
                'ppr.ratingValue',
                'ppr.datePublished',
                'ppr.verified',
                'ppr.replyFirstname',
                'ppr.replyLastname',
                'ppr.replyText',
                'ppr.helpful'
            ))->order('ppr.datePublished DESC')->where($where);

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }
}

