<?php

/**
 *
 * Class Widgets_Pagerating_Pagerating
 */
class Widgets_Pagerating_Pagerating extends Widgets_Abstract
{

    /**
     * Website config
     *
     * @var null
     */
    protected $_websiteHelper = null;

    /**
     * translation
     *
     * @var null
     */
    protected $_translator = null;

    protected $_cacheable = false;

    protected $_pageId = null;

    protected $_ratingData = array();

    protected $_ratingReviewData = array();

    /**
     * Prepare cart content
     */
    protected function  _init()
    {
        parent::_init();
        $this->_view = new Zend_View(array('scriptPath' => __DIR__ . '/views/'));

        $this->_websiteHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('website');
        $this->_view->websiteUrl = $this->_websiteHelper->getUrl();
        $this->_translator = Zend_Registry::get('Zend_Translate');

        $this->_pageId = intval($this->_toasterOptions['id']);

        $ratingData = Pagerating_Mappers_RatingMapper::getInstance()->find(
            array('pageId = ?' => $this->_pageId)
        );

        if($ratingData instanceof Pagerating_Models_Rating) {
            $this->_ratingData = $ratingData;
        }

    }

    protected function _load()
    {
        if (!isset($this->_options[0])) {
            return '';
        }

        $widgetName = '{$pagerating:' . implode(':', $this->_options) . '}';

        if (in_array('reviewitem', $this->_options, true)) {
            unset($this->_options[array_search('reviewitem', $this->_options, true)]);
            if (isset($this->_options['review']) && is_numeric($this->_options['reviewId'])) {
                $option = strtolower(array_shift($this->_options));
                $rendererName = '_renderReviewitem' . ucfirst($option);
                if (method_exists($this, $rendererName)) {
                    $this->_ratingReviewData = $this->_options['review'];

                    return $this->$rendererName();
                }
            } else {
                return $widgetName;
            }
        } else {
            if($this->_ratingData instanceof Pagerating_Models_Rating) {
                $option = strtolower(array_shift($this->_options));
                $methodName = '_render' . ucfirst(strtolower(trim($option)));
                if (method_exists($this, $methodName)) {
                    return $this->$methodName();
                }
            }
        }

        return '';

    }

    /**
     * Return rating value
     *
     * @return string
     */
    protected function _renderRatingValue()
    {
        return $this->_ratingData->getRatingValue();
    }

    /**
     * Return rating count
     *
     * @return string
     */
    protected function _renderRatingCount()
    {
        return $this->_ratingData->getRatingCount();
    }

    /**
     * Return rating total points
     *
     * @return string
     */
    protected function _renderTotalPoints()
    {
        return $this->_ratingData->getTotalPoints();
    }

    /**
     * Return review item name
     *
     * @return string
     */
    protected function _renderReviewitemName()
    {
        return htmlspecialchars($this->_ratingReviewData['name'], ENT_QUOTES, 'UTF-8');
    }

    /**
     * Return review item author
     *
     * @return string
     */
    protected function _renderReviewitemAuthor()
    {
        return htmlspecialchars($this->_ratingReviewData['author'], ENT_QUOTES, 'UTF-8');
    }

    /**
     * Return review item email
     *
     * @return string
     */
    protected function _renderReviewitemEmail()
    {
        return htmlspecialchars($this->_ratingReviewData['email'], ENT_QUOTES, 'UTF-8');
    }

    /**
     * Return review item description
     *
     * @return string
     */
    protected function _renderReviewitemDescription()
    {
        return htmlspecialchars($this->_ratingReviewData['description'], ENT_QUOTES, 'UTF-8');
    }

    /**
     * Return review item ratingValue
     *
     * @return string
     */
    protected function _renderReviewitemRatingValue()
    {
        if(in_array('img', $this->_options)) {
            $this->_view->rating = $this->_ratingReviewData;

            return $this->_view->render('stars.phtml');
        } else {
            return $this->_ratingReviewData['ratingValue'];
        }
    }

    /**
     * Return review item datePublished
     *
     * @return string
     */
    protected function _renderReviewitemDatePublished()
    {
        return date('Y-m-d', strtotime($this->_ratingReviewData['datePublished']));
    }

    /**
     * Return review item verified
     *
     * @return string
     *
     */
    protected function _renderReviewitemVerified()
    {
        return $this->_ratingReviewData['verified'];
    }

    /**
     * Return review item replyFirstname
     *
     * @return string
     */
    protected function _renderReviewitemReplyFirstname()
    {
        return htmlspecialchars($this->_ratingReviewData['replyFirstname'], ENT_QUOTES, 'UTF-8');
    }

    /**
     * Return review item replyLastname
     *
     * @return string
     */
    protected function _renderReviewitemReplyLastname()
    {
        return htmlspecialchars($this->_ratingReviewData['replyLastname'], ENT_QUOTES, 'UTF-8');
    }

    /**
     * Return review item replyText
     *
     * @return string
     */
    protected function _renderReviewitemReplyText()
    {
        return htmlspecialchars($this->_ratingReviewData['replyText'], ENT_QUOTES, 'UTF-8');
    }

    /**
     * Return review item helpful
     *
     * @return string
     */
    protected function _renderReviewitemHelpful()
    {
        return $this->_ratingReviewData['helpful'];
    }

}
